@@ -580,6 +580,12 @@ class QueryTestCase(TestCase):
         self.assertEquals(Person.objects.using('other').count(), 0)
         self.assertEquals(Pet.objects.using('other').count(), 0)
 
+    def test_foreign_key_validation(self):
+        "ForeignKey.validate() uses the correct database"
+        mickey = Person.objects.using('other').create(name="Mickey")
+        pluto = Pet.objects.using('other').create(name="Pluto", owner=mickey)
+        self.assertEquals(None, pluto.full_clean())
+
     def test_o2o_separation(self):
         "OneToOne fields are constrained to a single database"
         # Create a user and profile on the default database
@@ -852,10 +858,10 @@ class QueryTestCase(TestCase):
         "test the raw() method across databases"
         dive = Book.objects.using('other').create(title="Dive into Python",
             published=datetime.date(2009, 5, 4))
-        val = Book.objects.db_manager("other").raw('SELECT id FROM "multiple_database_book"')
+        val = Book.objects.db_manager("other").raw('SELECT id FROM multiple_database_book')
         self.assertEqual(map(lambda o: o.pk, val), [dive.pk])
 
-        val = Book.objects.raw('SELECT id FROM "multiple_database_book"').using('other')
+        val = Book.objects.raw('SELECT id FROM multiple_database_book').using('other')
         self.assertEqual(map(lambda o: o.pk, val), [dive.pk])
 
     def test_select_related(self):
@@ -883,7 +889,35 @@ class QueryTestCase(TestCase):
         self.assertRaises(ValueError, str, qs.query)
 
         # Evaluating the query shouldn't work, either
-        self.assertRaises(ValueError, list, qs)
+        try:
+            for obj in qs:
+                pass
+            self.fail('Iterating over query should raise ValueError')
+        except ValueError:
+            pass
+
+    def test_related_manager(self):
+        "Related managers return managers, not querysets"
+        mark = Person.objects.using('other').create(name="Mark Pilgrim")
+
+        # extra_arg is removed by the BookManager's implementation of
+        # create(); but the BookManager's implementation won't get called
+        # unless edited returns a Manager, not a queryset
+        mark.book_set.create(title="Dive into Python",
+                             published=datetime.date(2009, 5, 4),
+                             extra_arg=True)
+
+        mark.book_set.get_or_create(title="Dive into Python",
+                                    published=datetime.date(2009, 5, 4),
+                                    extra_arg=True)
+
+        mark.edited.create(title="Dive into Water",
+                           published=datetime.date(2009, 5, 4),
+                           extra_arg=True)
+
+        mark.edited.get_or_create(title="Dive into Water",
+                                  published=datetime.date(2009, 5, 4),
+                                  extra_arg=True)
 
 class TestRouter(object):
     # A test router. The behaviour is vaguely master/slave, but the
@@ -1491,19 +1525,10 @@ class AuthTestCase(TestCase):
         self.old_routers = router.routers
         router.routers = [AuthRouter()]
 
-        # Redirect stdout to a buffer so we can test
-        # the output of a management command
-        self.old_stdout = sys.stdout
-        self.stdout = StringIO()
-        sys.stdout = self.stdout
-
     def tearDown(self):
         # Restore the 'other' database as an independent database
         router.routers = self.old_routers
 
-        # Restore stdout
-        sys.stdout = self.old_stdout
-
     def test_auth_manager(self):
         "The methods on the auth manager obey database hints"
         # Create one user using default allocation policy
@@ -1539,22 +1564,28 @@ class AuthTestCase(TestCase):
 
         # Check that dumping the default database doesn't try to include auth
         # because allow_syncdb prohibits auth on default
-        self.stdout.flush()
-        management.call_command('dumpdata', 'auth', format='json', database='default')
-        self.assertEquals(self.stdout.getvalue(), '[]\n')
+        new_io = StringIO()
+        management.call_command('dumpdata', 'auth', format='json', database='default', stdout=new_io)
+        command_output = new_io.getvalue().strip()
+        self.assertEqual(command_output, '[]')
 
         # Check that dumping the other database does include auth
-        self.stdout.flush()
-        management.call_command('dumpdata', 'auth', format='json', database='other')
-        self.assertTrue('alice@example.com' in self.stdout.getvalue())
+        new_io = StringIO()
+        management.call_command('dumpdata', 'auth', format='json', database='other', stdout=new_io)
+        command_output = new_io.getvalue().strip()
+        self.assertTrue('"email": "alice@example.com",' in command_output)
 
+_missing = object()
 class UserProfileTestCase(TestCase):
     def setUp(self):
-        self.old_auth_profile_module = getattr(settings, 'AUTH_PROFILE_MODULE', None)
+        self.old_auth_profile_module = getattr(settings, 'AUTH_PROFILE_MODULE', _missing)
         settings.AUTH_PROFILE_MODULE = 'multiple_database.UserProfile'
 
     def tearDown(self):
-        settings.AUTH_PROFILE_MODULE = self.old_auth_profile_module
+        if self.old_auth_profile_module is _missing:
+            del settings.AUTH_PROFILE_MODULE
+        else:
+            settings.AUTH_PROFILE_MODULE = self.old_auth_profile_module
 
     def test_user_profiles(self):
 
@@ -1570,11 +1601,31 @@ class UserProfileTestCase(TestCase):
         self.assertEquals(alice.get_profile().flavor, 'chocolate')
         self.assertEquals(bob.get_profile().flavor, 'crunchy frog')
 
+class AntiPetRouter(object):
+    # A router that only expresses an opinion on syncdb,
+    # passing pets to the 'other' database
+
+    def allow_syncdb(self, db, model):
+        "Make sure the auth app only appears on the 'other' db"
+        if db == 'other':
+            return model._meta.object_name == 'Pet'
+        else:
+            return model._meta.object_name != 'Pet'
+        return None
 
 class FixtureTestCase(TestCase):
     multi_db = True
     fixtures = ['multidb-common', 'multidb']
 
+    def setUp(self):
+        # Install the anti-pet router
+        self.old_routers = router.routers
+        router.routers = [AntiPetRouter()]
+
+    def tearDown(self):
+        # Restore the 'other' database as an independent database
+        router.routers = self.old_routers
+
     def test_fixture_loading(self):
         "Multi-db fixtures are loaded correctly"
         # Check that "Pro Django" exists on the default database, but not on other database
@@ -1612,6 +1663,14 @@ class FixtureTestCase(TestCase):
         except Book.DoesNotExist:
             self.fail('"The Definitive Guide to Django" should exist on both databases')
 
+    def test_pseudo_empty_fixtures(self):
+        "A fixture can contain entries, but lead to nothing in the database; this shouldn't raise an error (ref #14068)"
+        new_io = StringIO()
+        management.call_command('loaddata', 'pets', stdout=new_io, stderr=new_io)
+        command_output = new_io.getvalue().strip()
+        # No objects will actually be loaded
+        self.assertTrue("Installed 0 object(s) (of 2) from 1 fixture(s)" in command_output)
+
 class PickleQuerySetTestCase(TestCase):
     multi_db = True
 
@@ -1620,3 +1679,94 @@ class PickleQuerySetTestCase(TestCase):
             Book.objects.using(db).create(title='Dive into Python', published=datetime.date(2009, 5, 4))
             qs = Book.objects.all()
             self.assertEqual(qs.db, pickle.loads(pickle.dumps(qs)).db)
+
+class AttributeErrorRouter(object):
+    "A router to test the exception handling of ConnectionRouter"
+    def db_for_read(self, model, **hints):
+        raise AttributeError
+
+    def db_for_write(self, model, **hints):
+        raise AttributeError
+
+class RouterAttributeErrorTestCase(TestCase):
+    multi_db = True
+
+    def setUp(self):
+        self.old_routers = router.routers
+        router.routers = [AttributeErrorRouter()]
+
+    def tearDown(self):
+        router.routers = self.old_routers
+
+    def test_attribute_error_read(self):
+        "Check that the AttributeError from AttributeErrorRouter bubbles up"
+        router.routers = [] # Reset routers so we can save a Book instance
+        b = Book.objects.create(title="Pro Django",
+                                published=datetime.date(2008, 12, 16))
+        router.routers = [AttributeErrorRouter()] # Install our router
+        self.assertRaises(AttributeError, Book.objects.get, pk=b.pk)
+
+    def test_attribute_error_save(self):
+        "Check that the AttributeError from AttributeErrorRouter bubbles up"
+        dive = Book()
+        dive.title="Dive into Python"
+        dive.published = datetime.date(2009, 5, 4)
+        self.assertRaises(AttributeError, dive.save)
+
+    def test_attribute_error_delete(self):
+        "Check that the AttributeError from AttributeErrorRouter bubbles up"
+        router.routers = [] # Reset routers so we can save our Book, Person instances
+        b = Book.objects.create(title="Pro Django",
+                                published=datetime.date(2008, 12, 16))
+        p = Person.objects.create(name="Marty Alchin")
+        b.authors = [p]
+        b.editor = p
+        router.routers = [AttributeErrorRouter()] # Install our router
+        self.assertRaises(AttributeError, b.delete)
+
+    def test_attribute_error_m2m(self):
+        "Check that the AttributeError from AttributeErrorRouter bubbles up"
+        router.routers = [] # Reset routers so we can save our Book, Person instances
+        b = Book.objects.create(title="Pro Django",
+                                published=datetime.date(2008, 12, 16))
+        p = Person.objects.create(name="Marty Alchin")
+        router.routers = [AttributeErrorRouter()] # Install our router
+        self.assertRaises(AttributeError, setattr, b, 'authors', [p])
+
+class ModelMetaRouter(object):
+    "A router to ensure model arguments are real model classes"
+    def db_for_write(self, model, **hints):
+        if not hasattr(model, '_meta'):
+            raise ValueError
+
+class RouterModelArgumentTestCase(TestCase):
+    multi_db = True
+
+    def setUp(self):
+        self.old_routers = router.routers
+        router.routers = [ModelMetaRouter()]
+
+    def tearDown(self):
+        router.routers = self.old_routers
+
+    def test_m2m_collection(self):
+        b = Book.objects.create(title="Pro Django",
+                                published=datetime.date(2008, 12, 16))
+
+        p = Person.objects.create(name="Marty Alchin")
+        # test add
+        b.authors.add(p)
+        # test remove
+        b.authors.remove(p)
+        # test clear
+        b.authors.clear()
+        # test setattr
+        b.authors = [p]
+        # Test M2M collection (_collect_sub_objects() in Django <= 1.2.X)
+        b.delete()
+
+    def test_foreignkey_collection(self):
+        person = Person.objects.create(name='Bob')
+        pet = Pet.objects.create(owner=person, name='Wart')
+        # Test related FK collection (_collect_sub_objects() in Django <= 1.2.X)
+        person.delete()
